@@ -1,13 +1,13 @@
 import logging
 from datetime import datetime
-from logging import CRITICAL, DEBUG, ERROR, INFO, NOTSET, WARN
+from logging import CRITICAL, DEBUG, ERROR, INFO, NOTSET, WARNING
 from threading import Lock
 
 TRACE = 5
 _levelToName = {
     CRITICAL: "critical",
     ERROR: "error",
-    WARN: "warning",
+    WARNING: "warning",
     INFO: "info",
     DEBUG: "debug",
     TRACE: "trace",
@@ -60,7 +60,7 @@ class StringFormatter(logging.Formatter):
         return super().format(record)
 
 
-def basicConfig(**kwargs):
+def basicConfig(**kwargs) -> logging.StreamHandler:
     with _config_lock:
         filename = kwargs.get("filename")
         if filename:
@@ -82,6 +82,8 @@ def basicConfig(**kwargs):
         if level is not None:
             root.setLevel(level)
 
+    return handler
+
 
 BASIC_FORMAT = "[{name}][{levelname}] {message}"
 FORMAT_STYLE = "{"
@@ -91,7 +93,7 @@ REMOVE_BASE = ["streamlink", "streamlink_cli"]
 
 logging.setLoggerClass(StreamlinkLogger)
 root = logging.getLogger("streamlink")
-root.setLevel(logging.WARNING)
+root.setLevel(WARNING)
 
 
-__all__ = ["StreamlinkLogger", "basicConfig", "root", "levels"]
+__all__ = ["StreamlinkLogger", "TRACE", "basicConfig", "root", "levels"]
