@@ -25,7 +25,19 @@ from streamlink_cli.main import (
     setup_config_args
 )
 from streamlink_cli.output import FileOutput, PlayerOutput
-from tests.plugin.testplugin import TestPlugin as FakePlugin
+from tests.plugin.testplugin import TestPlugin as _TestPlugin
+
+
+class FakePlugin(_TestPlugin):
+    module = "fake"
+    arguments = []
+    _streams = {}
+
+    def streams(self, *args, **kwargs):
+        return self._streams
+
+    def _get_streams(self):  # pragma: no cover
+        pass
 
 
 class TestCLIMain(unittest.TestCase):
@@ -71,7 +83,7 @@ class TestCLIMain(unittest.TestCase):
             "best": c
         }
         self.assertEqual(
-            format_valid_streams(FakePlugin, streams),
+            format_valid_streams(_TestPlugin, streams),
             ", ".join([
                 "audio",
                 "720p (worst)",
@@ -87,7 +99,7 @@ class TestCLIMain(unittest.TestCase):
             "best-unfiltered": c
         }
         self.assertEqual(
-            format_valid_streams(FakePlugin, streams),
+            format_valid_streams(_TestPlugin, streams),
             ", ".join([
                 "audio",
                 "720p (worst-unfiltered)",
@@ -102,10 +114,9 @@ class TestCLIMainJsonAndStreamUrl(unittest.TestCase):
     def test_handle_stream_with_json_and_stream_url(self, console, args):
         stream = Mock()
         streams = dict(best=stream)
+
         plugin = FakePlugin("")
-        plugin.module = "fake"
-        plugin.arguments = []
-        plugin.streams = Mock(return_value=streams)
+        plugin._streams = streams
 
         handle_stream(plugin, streams, "best")
         self.assertEqual(console.msg.mock_calls, [])
@@ -138,12 +149,11 @@ class TestCLIMainJsonAndStreamUrl(unittest.TestCase):
     def test_handle_url_with_json_and_stream_url(self, console, args):
         stream = Mock()
         streams = dict(worst=Mock(), best=stream)
-        plugin = FakePlugin("")
-        plugin.module = "fake"
-        plugin.arguments = []
-        plugin.streams = Mock(return_value=streams)
 
-        with patch("streamlink_cli.main.streamlink", resolve_url=Mock(return_value=plugin)):
+        class _FakePlugin(FakePlugin):
+            _streams = streams
+
+        with patch("streamlink_cli.main.streamlink", resolve_url=Mock(return_value=(_FakePlugin, ""))):
             handle_url()
             self.assertEqual(console.msg.mock_calls, [])
             self.assertEqual(console.msg_json.mock_calls, [call(
@@ -174,43 +184,57 @@ class TestCLIMainJsonAndStreamUrl(unittest.TestCase):
 
 
 class TestCLIMainCheckFileOutput(unittest.TestCase):
-    @patch("streamlink_cli.main.os.path.isfile", Mock(return_value=False))
+    @staticmethod
+    def mock_path(path, is_file=True):
+        return Mock(
+            spec=Path(path),
+            is_file=Mock(return_value=is_file),
+            __str__=Mock(return_value=path)
+        )
+
     def test_check_file_output(self):
-        output = check_file_output("foo", False)
+        path = self.mock_path("foo", is_file=False)
+        output = check_file_output(path, False)
         self.assertIsInstance(output, FileOutput)
-        self.assertEqual(output.filename, "foo")
+        self.assertIs(output.filename, path)
 
-    @patch("streamlink_cli.main.os.path.isfile", Mock(return_value=True))
     def test_check_file_output_exists_force(self):
-        output = check_file_output("foo", True)
+        path = self.mock_path("foo", is_file=True)
+        output = check_file_output(path, True)
         self.assertIsInstance(output, FileOutput)
-        self.assertEqual(output.filename, "foo")
+        self.assertIs(output.filename, path)
 
-    @patch("streamlink_cli.main.console", Mock(ask=Mock(return_value="y")))
-    @patch("streamlink_cli.main.os.path.isfile", Mock(return_value=True))
+    @patch("streamlink_cli.main.console")
     @patch("streamlink_cli.main.sys")
-    def test_check_file_output_exists_ask_yes(self, mock_sys: Mock):
+    def test_check_file_output_exists_ask_yes(self, mock_sys: Mock, mock_console: Mock):
         mock_sys.stdin.isatty.return_value = True
-        output = check_file_output("foo", False)
+        mock_console.ask = Mock(return_value="y")
+        path = self.mock_path("foo", is_file=True)
+        output = check_file_output(path, False)
+        self.assertEqual(mock_console.ask.call_args_list, [call("File foo already exists! Overwrite it? [y/N] ")])
         self.assertIsInstance(output, FileOutput)
-        self.assertEqual(output.filename, "foo")
+        self.assertIs(output.filename, path)
 
-    @patch("streamlink_cli.main.console", Mock(ask=Mock(return_value="N")))
-    @patch("streamlink_cli.main.os.path.isfile", Mock(return_value=True))
+    @patch("streamlink_cli.main.console")
     @patch("streamlink_cli.main.sys")
-    def test_check_file_output_exists_ask_no(self, mock_sys: Mock):
+    def test_check_file_output_exists_ask_no(self, mock_sys: Mock, mock_console: Mock):
         mock_sys.stdin.isatty.return_value = True
         mock_sys.exit.side_effect = SystemExit
+        mock_console.ask = Mock(return_value="N")
+        path = self.mock_path("foo", is_file=True)
         with self.assertRaises(SystemExit):
-            check_file_output("foo", False)
+            check_file_output(path, False)
+        self.assertEqual(mock_console.ask.call_args_list, [call("File foo already exists! Overwrite it? [y/N] ")])
 
-    @patch("streamlink_cli.main.os.path.isfile", Mock(return_value=True))
+    @patch("streamlink_cli.main.console")
     @patch("streamlink_cli.main.sys")
-    def test_check_file_output_exists_notty(self, mock_sys: Mock):
+    def test_check_file_output_exists_notty(self, mock_sys: Mock, mock_console: Mock):
         mock_sys.stdin.isatty.return_value = False
         mock_sys.exit.side_effect = SystemExit
+        path = self.mock_path("foo", is_file=True)
         with self.assertRaises(SystemExit):
-            check_file_output("foo", False)
+            check_file_output(path, False)
+        self.assertEqual(mock_console.ask.call_args_list, [])
 
 
 class TestCLIMainCreateOutput(unittest.TestCase):
@@ -239,74 +263,81 @@ class TestCLIMainCreateOutput(unittest.TestCase):
         self.assertIsInstance(output, PlayerOutput)
         self.assertEqual(output.title, "foo - bar")
 
-    @patch("streamlink_cli.main.os.path.isfile")
     @patch("streamlink_cli.main.args")
-    @patch("streamlink_cli.main.console", Mock())
-    def test_create_output_file_output(self, args: Mock, mock_isfile):
+    @patch("streamlink_cli.main.check_file_output")
+    def test_create_output_file_output(self, mock_check_file_output: Mock, args: Mock):
         formatter = Formatter({})
+        mock_check_file_output.side_effect = lambda path, force: FileOutput(path)
         args.output = "foo"
+        args.stdout = None
+        args.record = None
+        args.record_and_pipe = None
         args.force = False
         args.fs_safe_rules = None
-        mock_isfile.return_value = False
 
         output = create_output(formatter)
+        self.assertEqual(mock_check_file_output.call_args_list, [call(Path("foo"), False)])
         self.assertIsInstance(output, FileOutput)
-        self.assertEqual(output.filename, "foo")
-        self.assertEqual(output.fd, None)
-        self.assertEqual(output.record, None)
+        self.assertEqual(output.filename, Path("foo"))
+        self.assertIsNone(output.fd)
+        self.assertIsNone(output.record)
 
     @patch("streamlink_cli.main.args")
-    @patch("streamlink_cli.main.console", Mock())
     def test_create_output_stdout(self, args: Mock):
         formatter = Formatter({})
         args.output = None
         args.stdout = True
+        args.record = None
+        args.record_and_pipe = None
 
         output = create_output(formatter)
         self.assertIsInstance(output, FileOutput)
-        self.assertEqual(output.filename, None)
-        self.assertEqual(output.fd, stdout)
-        self.assertEqual(output.record, None)
+        self.assertIsNone(output.filename)
+        self.assertIs(output.fd, stdout)
+        self.assertIsNone(output.record)
 
         args.output = "-"
         args.stdout = False
         output = create_output(formatter)
         self.assertIsInstance(output, FileOutput)
-        self.assertEqual(output.filename, None)
-        self.assertEqual(output.fd, stdout)
-        self.assertEqual(output.record, None)
+        self.assertIsNone(output.filename)
+        self.assertIs(output.fd, stdout)
+        self.assertIsNone(output.record)
 
-    @patch("streamlink_cli.main.os.path.isfile")
     @patch("streamlink_cli.main.args")
-    @patch("streamlink_cli.main.console", Mock())
-    def test_create_output_record_and_pipe(self, args: Mock, mock_isfile: Mock):
+    @patch("streamlink_cli.main.check_file_output")
+    def test_create_output_record_and_pipe(self, mock_check_file_output: Mock, args: Mock):
         formatter = Formatter({})
+        mock_check_file_output.side_effect = lambda path, force: FileOutput(path)
         args.output = None
         args.stdout = None
         args.record_and_pipe = "foo"
+        args.force = False
         args.fs_safe_rules = None
-        mock_isfile.return_value = False
 
         output = create_output(formatter)
+        self.assertEqual(mock_check_file_output.call_args_list, [call(Path("foo"), False)])
         self.assertIsInstance(output, FileOutput)
-        self.assertEqual(output.filename, None)
-        self.assertEqual(output.fd, stdout)
+        self.assertIsNone(output.filename)
+        self.assertIs(output.fd, stdout)
         self.assertIsInstance(output.record, FileOutput)
-        self.assertEqual(output.record.filename, "foo")
-        self.assertEqual(output.record.fd, None)
-        self.assertEqual(output.record.record, None)
+        self.assertEqual(output.record.filename, Path("foo"))
+        self.assertIsNone(output.record.fd)
+        self.assertIsNone(output.record.record)
 
     @patch("streamlink_cli.main.args")
-    @patch("streamlink_cli.main.console", Mock())
+    @patch("streamlink_cli.main.check_file_output")
     @patch("streamlink_cli.main.DEFAULT_STREAM_METADATA", {"title": "bar"})
-    def test_create_output_record(self, args: Mock):
+    def test_create_output_record(self, mock_check_file_output: Mock, args: Mock):
         formatter = Formatter({
             "author": lambda: "foo"
         })
+        mock_check_file_output.side_effect = lambda path, force: FileOutput(path)
         args.output = None
         args.stdout = None
         args.record = "foo"
         args.record_and_pipe = None
+        args.force = False
         args.fs_safe_rules = None
         args.title = None
         args.url = "URL"
@@ -319,18 +350,18 @@ class TestCLIMainCreateOutput(unittest.TestCase):
         self.assertIsInstance(output, PlayerOutput)
         self.assertEqual(output.title, "URL")
         self.assertIsInstance(output.record, FileOutput)
-        self.assertEqual(output.record.filename, "foo")
-        self.assertEqual(output.record.fd, None)
-        self.assertEqual(output.record.record, None)
+        self.assertEqual(output.record.filename, Path("foo"))
+        self.assertIsNone(output.record.fd)
+        self.assertIsNone(output.record.record)
 
         args.title = "{author} - {title}"
         output = create_output(formatter)
         self.assertIsInstance(output, PlayerOutput)
         self.assertEqual(output.title, "foo - bar")
         self.assertIsInstance(output.record, FileOutput)
-        self.assertEqual(output.record.filename, "foo")
-        self.assertEqual(output.record.fd, None)
-        self.assertEqual(output.record.record, None)
+        self.assertEqual(output.record.filename, Path("foo"))
+        self.assertIsNone(output.record.fd)
+        self.assertIsNone(output.record.record)
 
     @patch("streamlink_cli.main.args")
     @patch("streamlink_cli.main.console")
@@ -361,7 +392,7 @@ class TestCLIMainHandleStream(unittest.TestCase):
         args.player_continuous_http = False
         mock_output_stream.return_value = True
 
-        plugin = FakePlugin("")
+        plugin = _TestPlugin("")
         plugin.author = "AUTHOR"
         plugin.category = "CATEGORY"
         plugin.title = "TITLE"
@@ -389,7 +420,7 @@ class TestCLIMainSetupConfigArgs(unittest.TestCase):
         def resolve_url(name):
             if name == "noplugin":
                 raise NoPluginError()
-            return Mock(module="testplugin")
+            return Mock(module="testplugin"), name
 
         session = Mock()
         session.resolve_url.side_effect = resolve_url
